/* 
 Converter: main.js
 
 Copyright 2009 John Bestoso. All rights reserved.
 Modified by Joakim Andersson, 2012.
 */

//
// Function: load()
// Called by HTML body element's onload event when the widget is ready to start
//
function load()
{
    dashcode.setupParts();
}

//
// Function: remove()
// Called when the widget has been removed from the Dashboard
//
function remove()
{
    // Stop any timers to prevent CPU usage
    // Remove any preferences as needed
    // widget.setPreferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
}

//
// Function: hide()
// Called when the widget has been hidden
//
function hide()
{
    // Stop any timers to prevent CPU usage
}

//
// Function: show()
// Called when the widget has been shown
//
function show()
{
    // Restart any timers that were stopped on hide
}

//
// Function: sync()
// Called when the widget has been synchronized with .Mac
//
function sync()
{
    // Retrieve any preference values that you need to be synchronized here
    // Use this for an instance key's value:
    // instancePreferenceValue = widget.preferenceForKey(null, dashcode.createInstancePreferenceKey("your-key"));
    //
    // Or this for global key's value:
    // globalPreferenceValue = widget.preferenceForKey(null, "your-key");
}

//
// Function: showBack(event)
// Called when the info button is clicked to show the back of the widget
//
// event: onClick event from the info button
//
function showBack(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToBack");
    }

    front.style.display = "none";
    back.style.display = "block";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

//
// Function: showFront(event)
// Called when the done button is clicked from the back of the widget
//
// event: onClick event from the done button
//
function showFront(event)
{
    var front = document.getElementById("front");
    var back = document.getElementById("back");

    if (window.widget) {
        widget.prepareForTransition("ToFront");
    }

    front.style.display="block";
    back.style.display="none";

    if (window.widget) {
        setTimeout('widget.performTransition();', 0);
    }
}

if (window.widget) {
    widget.onremove = remove;
    widget.onhide = hide;
    widget.onshow = show;
    widget.onsync = sync;
}

//
// Function: updateValuesDecimal(event)
// Called when a value is entered into the decimal field.
//
// event: onkeyup event from decimalField
//

function updateValuesDecimal(event)
{
    // Grab the text fields
    var dec = document.getElementById("decimalField");
    var hex = document.getElementById("hexField");
    var bin = document.getElementById("binField");
    var oct = document.getElementById("octField");
    
    var n = parseInt(dec.value, 10);
    
    if(isNaN(n) || dec.value.match(/[^\d]/)) {
        hex.value = bin.value = oct.value = '';
    } else {
        // Update the hex value
        hex.value = convertDH(n);
    
        // Update the binary value
        bin.value = convertDB(n);
        
        // Update the octal value
        oct.value = convertDO(n);
    }
 
}

//
// Function: updateValuesHex(event)
// Called when a value is entered into the hex field.
//
// event: onkeyup event from hexField
//

function updateValuesHex(event)
{
    // Grab the text fields
    var dec = document.getElementById("decimalField");
    var hex = document.getElementById("hexField");
    var bin = document.getElementById("binField");
    var oct = document.getElementById("octField");
    
    var n = parseInt(hex.value.replace(/^0x/, ''), 16);
    
    if(isNaN(n) || hex.value.replace(/^0x/, '').match(/[^\da-fA-F]/)) {
        dec.value = bin.value = oct.value = '';
    } else {
        // Update the decimal value
        dec.value = n;
    
        // Update the binary value
        bin.value = convertDB(n);
        
        // Update the octal value
        oct.value = convertDO(n);
    }
 
}

//
// Function: updateValuesBin(event)
// Called when a value is entered into the binary field.
//
// event: onkeyup event from binField
//

function updateValuesBin(event)
{
    // Grab the text fields
    var dec = document.getElementById("decimalField");
    var hex = document.getElementById("hexField");
    var bin = document.getElementById("binField");
    var oct = document.getElementById("octField");
    
    var n = parseInt(bin.value.replace(/^0b/, ''), 2);
    
    if(isNaN(n) || bin.value.replace(/^0b/, '').match(/[^0-1]/)) {
        dec.value = hex.value = oct.value = '';
    } else {
        // Update the decimal value
        dec.value = n;
    
        // Update the hex value
        hex.value = convertDH(n);
        
        // Update the octal value
        oct.value = convertDO(n);
    }
 
}

//
// Function: updateValuesOct(event)
// Called when a value is entered into the octal field.
//
// event: onkeyup event from octField
//

function updateValuesOct(event)
{
    // Grab the text fields
    var dec = document.getElementById("decimalField");
    var hex = document.getElementById("hexField");
    var bin = document.getElementById("binField");
    var oct = document.getElementById("octField");
    
    var n = parseInt(oct.value, 8);
    
    if(isNaN(n) || oct.value.match(/[^0-7]/)) {
        dec.value = hex.value = bin.value = '';
    } else {
        // Update the decimal value
        dec.value = n;
    
        // Update the hex value
        hex.value = convertDH(n);
    
        // Update the binary value
        bin.value = convertDB(n);
    }
}

//
// Function: convertDH(n)
// Converts a decimal number n to hexadecimal, and returns it
//

function convertDH(n)
{

    // Special case
    if(n == 0)
        return '0x0';

    // Declare the result string
    var result = "";
    var remainder;
    
    while(n > 0) {
        
        // Find the remainder and divide by 16
        remainder = n % 16;
        n -= remainder;
        n /= 16;
        
        // Convert to a,b,c etc if remainder > 16
        switch(remainder) {
            
            case 15:
                remainder = "F";
                break;
            case 14:
                remainder = "E";
                break;
            case 13:
                remainder = "D";
                break;
            case 12:
                remainder = "C";
                break;
            case 11:
                remainder = "B";
                break;
            case 10:
                remainder = "A";
                break;
        }
                
        // Prepend the latest to our result string
        result = remainder + result;
    }
    
    return '0x' + result;
}
    
        
//
// Function: convertDB(n)
// Converts a decimal number n to binary, and returns it
//

function convertDB(n)
{

    // Special case. So sue me
    if(n == 0)
        return '0b0';

    // Declare result string and remainder var
    var result = "";
    var remainder;
    
    while(n > 0) {
    
        // Get remainder and divide by 2
        remainder = n % 2;
        n -= remainder;
        n /= 2;
        
        // Prepend to the result
        result = remainder + result;
        
    }
    
    return '0b' + result;
    
}
    
        
//
// Function: convertDO(n)
// Converts a decimal number n to octal, and returns it
//

function convertDO(n)
{

    // Special case. So sue me
    if(n == 0)
        return '00';

    // Declare result string and remainder var
    var result = "";
    var remainder;
    
    while(n > 0) {
    
        // Get remainder and divide by 8
        remainder = n % 8;
        n -= remainder;
        n /= 8;
        
        // Prepend to the result
        result = remainder + result;
        
    }
    
    return '0' + result;
    
}